﻿module DocumentationMenu {

    interface NodeData {
        name: string;
        description: string;
        href: string;
        expanded: boolean;
        selected: boolean;
        nodes: NodeData[];
    }
    interface Tree {
        nodes: NodeData[];
    }
    interface Input {
        tree: Tree;
    }

    class NodeModel {

        isExpanded = ko.observable(false);
        isSelected = ko.observable(false);
        description = ko.observable('');
        name = ko.observable('');
        href = ko.observable('');
        nodes = ko.observable<NodeModel[]>();
        nodeCssClass: KnockoutComputed<object>;

        constructor(data: NodeData) {
            if (data == null)
                throw "null data";

            this.isExpanded(data.expanded);
            this.isSelected(data.selected);
            this.description(data.description);
            this.name(data.name);
            this.href(data.href);
            this.nodeCssClass = ko.pureComputed(
                () => {
                    return {
                        'minus-sign': this.isExpanded() && this.nodes().length > 0,
                        'plus-sign': !this.isExpanded() && this.nodes().length > 0,
                        'no-icon': this.nodes().length === 0
                    };
                }
            );

            this.nodes(data.nodes.makeIterable().select(n => new NodeModel(n)).toArray());
        }

        toggleVisibility(): boolean {
            if (this.nodes().length == 0)
                return this.visitLink();

            this.isExpanded(!this.isExpanded());
            return false;
        };

        visitLink(): boolean {
            window.location.href = this.href();
            return false;
        }
    }

    class TreeModel {
        nodes = ko.observable<NodeModel[]>();
        constructor(tree: Tree) {
            this.nodes(tree.nodes.makeIterable().select(n => new NodeModel(n)).toArray());
        }
    }

    class PageModel {

        tree = ko.observable<TreeModel>();

        constructor(input: Input) {
            this.tree(new TreeModel(input.tree));
        }
    }

    export function Init() {

        $.ajax("/tree.json", {
            type: "GET",
            cache: false,
            contentType: "application/json",
            data: {
                path: window.location.pathname
            },
            success: (d: Input) => {
                var model = new PageModel(d);
                var root = document.getElementById("nav-tree");
                ko.applyBindings(model, root);
            }
        })
    }
}

$(() => DocumentationMenu.Init());